!===========
program main
!===========
!
! Appel aux modules
        use mpi
        use mod_dataproc
        use mod_data
        use mod_coefmat
        use mod_secmbre
        use mod_gc
        use mod_normvect
        use mod_imprim
!
        implicit none
!
! Declaration des tableaux dynamiques
        real(kind=8), dimension(:,:), allocatable :: A
        real(kind=8), dimension(:), allocatable :: b, u, u0, uexact
        real(kind=8), dimension(:), allocatable :: ug, ugexact
!
! Declaration des autres variables
        real(kind=8) :: pi, pi2, xi
        integer :: nx, i, n
        character(len=99) :: charnitert, charniterx
        character(len=99) :: charnxg
        character(len=99), dimension(0:60) :: charnxl
        integer, dimension(0:60) :: rcounts, displs
!
! Initialisation de MPI
        call MPI_INIT (code)
        call MPI_COMM_RANK (MPI_COMM_WORLD, moi, code)
        call MPI_COMM_SIZE (MPI_COMM_WORLD, nprocs, code)
!
! Lecture des donnees
        if (moi==0) then
        open (unit = 10, file = 'datafile', status = 'old')
        read (10,'(a99)') dirloc
        read (10,*) temps, dt
        read (10,*) epst
        read (10,*) nx
        read (10,*) a1, a2, a3
        read (10,*) epsx, niterxmax
        read (10,*) nimp
        close (10)
        end if
!
! Distribution des donnees par moi=0 sur tous les autres processus
        call MPI_BCAST (dirloc, 99, MPI_CHARACTER, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (temps, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (dt, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (epst, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (nx, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (a1, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (a2, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (a3, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (epsx, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (niterxmax, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (nimp, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, code)
!
! Pas de maillage sur le domaine global en espace
        dx = 1./float(nx)
!
! Nombre global de noeuds interieurs 1 <= i <= nxg
        nxg = nx-1
        if (moi==0) write(charnxg,*) nxg
        call MPI_BCAST (charnxg, 99, MPI_CHARACTER, 0, MPI_COMM_WORLD, code)
!
! Nombre local de noeuds interieurs par sous-domaine 1 <= i <= nxl
        nxl1 = nint (float(nxg)/float(nprocs))
        nxl2 = nxg - (nxl1*(nprocs-1))
        nxl = nxl1
        if (moi==nprocs-1) nxl = nxl2
        write(charnxl(moi),*) nxl
        print'("moi=",i3, "   nbre global noeuds="a, "   nbre local noeuds="a)' &
              , moi, trim(adjustl(charnxg)), trim(adjustl(charnxl(moi)))
!
       if (moi==0) then
        do i = 0, nprocs-2
         rcounts(i)=nxl1
         displs(i)=i*nxl1
        end do
        i = nprocs-1
        rcounts(i)=nxl2
        displs(i)=i*nxl1
       end if
!
! Allocation dynamique des tableaux
        allocate (A(3,nxl), b(nxl), u(nxl), u0(nxl), uexact(nxl))
        if (moi==0) allocate (ug(nxg), ugexact(nxg))
!
! Solution analytique
        pi = acos(-1.d0)
        pi2 = 1./(pi**2)
        do i = 1, nxl
         xi = (i+(nxl1*moi))*dx
       ! uexact(i) = pi2*sin(pi*xi)
         uexact(i) = 6*xi*(1-xi)
       ! uexact(i) = xi*(1-xi)*exp(-xi)
        end do
!
        call MPI_GATHERV (uexact, nxl, MPI_DOUBLE_PRECISION, &
                          ugexact, rcounts, displs, MPI_DOUBLE_PRECISION, &
                          0, MPI_COMM_WORLD, code)
!
! Matrice du systeme lineaire
        call coefmat (A)
!
! Nombre iterations maximun en temps
        nt = int(temps/dt)
!
        algo = 'methode iterative du gradient conjugue'
! ===============
        u = 0.d0
! Boucle en temps
        do n = 1, nt
! ===============
!
          nitert = n
          u0 = u
          call secmbre (u0, b)
          u = gc (A, b, u0)
          normerrsol = dsqrt(dx) * normL2 (u-uexact)
          normerrsta = dsqrt(dx) * (normL2 (u-u0))/dt
!
          call MPI_GATHERV (u, nxl, MPI_DOUBLE_PRECISION, ug, rcounts, displs, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
!
          if (moi==0) then
           write(charnitert,*) nitert
           write(charniterx,*) niterx
          end if
          call MPI_BCAST (charnitert, 99, MPI_CHARACTER, 0, MPI_COMM_WORLD, code)
          call MPI_BCAST (charniterx, 99, MPI_CHARACTER, 0, MPI_COMM_WORLD, code)
!
          if ((mod(nitert,nimp)==0)) then
           print'("moi=",i3, "  niter.temps="a, "  niter.gc="a, " normresx.gc=",d12.7, "  err.sol=",d12.7, " err.sta=",d12.7)' &
                 , moi, trim(adjustl(charnitert)), trim(adjustl(charniterx)), normresx, normerrsol, normerrsta
           if (moi==0) call imprim (ug, ugexact)
          end if
!
          if (normerrsta<=epst) exit
!
! ===================
        end do
! Fin boucle en temps
! ===================
!
        print'("moi=",i3, "   FIN CALCUL"  "   nbre.iter.temps a convergence="a, "   err.sol=",d12.7, "   err.sta=",d12.7)' &
              , moi, trim(adjustl(charnitert)), normerrsol, normerrsta
        if (moi==0) call imprim (ug, ugexact)
!
! Desallocation des tableaux
        deallocate (A, b, u, u0, uexact)
        if (moi==0) deallocate (ug, ugexact)
!
! Fin de MPI
        call MPI_FINALIZE (code)

!
!===============
end program main
!===============
