!================
module mod_matvec
!================
!
!=====================
interface operator (*)
!=====================
!
        module procedure matvec
!
!============
end interface
!============
!
contains
!
!=====================
function matvec (A, v)
!=====================
!
        use mpi
        use mod_dataproc
!
        implicit none
!
        real(kind=8), dimension(:,:), intent(in) :: A
        real(kind=8), dimension(:), intent(in) :: v
        real(kind=8), dimension(size(v)) :: matvec
        integer :: n, i
        integer :: sid1, sidn, rid1, ridn
        integer, dimension (MPI_STATUS_SIZE) :: status
        real (kind = 8) :: west, est
!
        n = size(v)
!
! Buffers
        west = v(n)
        est = v(1)
!
! Echanges non bloquants
        if (moi /= 0) then
          call MPI_ISSEND (v(1), 1, MPI_DOUBLE_PRECISION, moi-1, 17, MPI_COMM_WORLD, sid1, code)
          call MPI_IRECV (west, 1, MPI_DOUBLE_PRECISION, moi-1, 17, MPI_COMM_WORLD, ridn, code)
        end if
        if (moi /= nprocs-1) then
          call MPI_ISSEND (v(n), 1, MPI_DOUBLE_PRECISION, moi+1, 17, MPI_COMM_WORLD, sidn, code)
          call MPI_IRECV (est , 1, MPI_DOUBLE_PRECISION, moi+1, 17, MPI_COMM_WORLD, rid1, code)
        end if
!
! Produit matrice-vecteur generique aux noeuds interieurs
        matvec(1) = A(2,1)*v(1 ) + A(3,1)*v(2)
        do i = 2, n-1
        matvec(i) = A(1,i)*v(i-1) + A(2,i)*v(i) + A(3,i)*v(i+1)
        end do
        matvec(n) = A(1,n)*v(n-1) + A(2,n)*v(n)
!
! Attente et liberation des processus pour le produit matrice-vecteur aux frontieres des sous-domaines
        if (moi /= 0) call MPI_WAIT (ridn, status, code)
        if (moi /= nprocs-1) call MPI_WAIT (rid1, status, code)
!
        if (moi /= 0) matvec(1) = matvec(1) + A(1,1) * west
        if (moi /= nprocs-1) matvec(n) = matvec(n) + A(3,n) * est
!
        if (moi /= 0) call MPI_WAIT (sid1, status, code)
        if (moi /= nprocs-1) call MPI_WAIT (sidn, status, code)
!
!==================
end function matvec
!==================
!
!====================
end module mod_matvec
!====================
